@@ -1,15 +1,18 @@
 from __future__ import annotations
-from typing import List, Optional
+from pathlib import Path
+from typing import List, Optional, Union, Dict
 
 import datetime as dt
 import requests
-from urllib.parse import urlparse, urljoin
-from base64 import b64encode, b64decode
+# from urllib.parse import urlparse, urljoin
+from base64 import b64decode #, b64encode
 
 try:
     from ._utils import Decorators, create_random_suffix
+    from ._base import OAuthClient
 except ImportError:
     from _utils import Decorators, create_random_suffix
+    from _base import OAuthClient
 
 __nettime__ = "6.0.1.19025"
 
@@ -61,67 +64,41 @@ class Query:
             return str(fields).replace("'", '"').replace(" ", "")
 
 
-class Client:
+class Client(OAuthClient):
+
+    __name__ = 'netTime6'
 
     def __init__(
         self,
-        url: str,
+        *,
+        url: Union[str, Path],
         username: str,
         pwd: str,
-        session: Optional[requests.Session] = None,
-        **kwargs
+        session: Optional[requests.Session] = None
     ) -> None:
         """ Create a connector with nettime app using recived parameters
 
         Args:
-            url (str): Nettime url. Eg "https://server-name:8091/"
+            url (Union[str, Path]): Nettime url. Eg `https://server-name:8091/`
             username (str): Nettime username
             pwd (str): Nettime password
+            session (Optional[requests.Session], optional):
+                Optional session handler. Defaults to None.
         """
+        super().__init__(
+            url=url,
+            username=username,
+            pwd=pwd,
+            session=session
+        )
 
-        self.client_url = urlparse(url)
-        self.username = username
-        self.pwd = b64encode(pwd.encode('utf-8'))
-        
-        # base headers
-        self.headers = {
-            "DNT": "1",
-            # "Content-Type": "application/json;charset=UTF-8",
-            "Accept-Encoding": "gzip,deflate",
-            # "Cookie": f"sessionID={self.access_token}; i18next=es",
-        }
-        
-        # manual session or None
-        self.session = session
+        self.headers = {"DNT": "1", "Accept-Encoding": "gzip,deflate"}
 
         ### None values
         self.access_token = None
         self.user_rol = None
         self.settings = None
 
-
-    def __str__(self):
-        return '{}{} en {}'.format(
-            f'{self.access_token} para ' if self.access_token else '',
-            self.username,
-            self.client_url.geturl()
-        )
-
-    def __repr__(self):
-        return "{}(url='{}', username='{}', pwd='{}')".format(
-            self.__class__.__name__,
-            self.client_url.geturl(),
-            self.username,
-            b64decode(self.pwd).decode('utf-8'),
-        )
-
-    def __enter__(self, *args, **kwargs) -> Client:
-        self.start_session()
-        return self
-
-    def __exit__(self, *args, **kwargs) -> None:
-        self.close_session()
-
     def refresh_headers(
         self,
         access_token: Optional[str] = None,
@@ -150,70 +127,11 @@ class Client:
                 "Content-Type": "application/json;charset=UTF-8"
             })
 
-    def start_session(self) -> None:
-        self.session = requests.Session()
-        self.session.headers.update(self.headers)
-
-        # login
-        self.login()
-
-    def refresh_session(self) -> None:
-        self.session.headers.update(self.headers)
-
-    def close_session(self):
-        self.logout()
-        self.session.close()
-        self.session = None
-
     @property
     def is_connected(self):
         """ Informs if client has headers and access_token. """
-
         return bool("Cookie" in self.headers and bool(self.access_token))
 
-    @Decorators.ensure_session
-    def get(
-        self,
-        path: str,
-        params: dict = None,
-        **kwargs
-    ) -> Union[Dict, List, requests.Response]:
-        """
-        Sends a GET request to nettime url.
-
-        :param path: path to add to URL for the new :class:`Request` object.
-        :param params: (optional) Dictionary, list of tuples or bytes to send
-            in the query string for the :class:`Request`.
-        :param \*\*kwargs: Optional arguments that ``request`` takes.
-        :return: json object
-        :rtype: json
-        """
-
-        # prepare url
-        url = urljoin(self.client_url.geturl(), path)
-
-        # consulting nettime
-        response = self.session.get(url=url, params=params, **kwargs)
-
-        # if session was closed, reconect client and try again
-        if response.status_code == 401:
-            self.relogin()
-            return self.get(path=path, params=params, **kwargs)
-
-        # raise if was an error
-        if response.status_code not in range(200, 300):
-            raise ConnectionError({
-                "status": response.status_code,
-                "detail": response.text
-            })
-        
-        # if request is stream type, return all response
-        if kwargs.get("stream"):
-            return response
-
-        # to json
-        return response.json()
-
     @Decorators.ensure_session
     def post(
         self,
@@ -236,39 +154,15 @@ class Client:
         :rtype: json
         """
 
-        # prepare url
-        url = urljoin(self.client_url.geturl(), path)
-
-        # consulting nettime
-        response = self.session.post(
-            url=url,
+        # to json -> json
+        json_response = super().post(
+            path=path,
             params=params,
             data=data,
             json=json,
             **kwargs
         )
 
-        # if session was closed, reconect client and try again
-        if response.status_code == 401:
-            self.relogin()
-            return self.post(
-                path=path,
-                params=params,
-                data=data,
-                json=json,
-                **kwargs
-            )
-
-        # raise if was an error
-        if response.status_code not in range(200, 300):
-            raise ConnectionError({
-                "status": response.status_code,
-                "detail": response.text
-            })
-
-        # to json -> json
-        json_response = response.json()
-
         # get task results if is generated
         if isinstance(json_response, dict) and \
                 json_response.get('taskId', None):
@@ -323,24 +217,24 @@ class Client:
         self.refresh_headers(remove_token=True, remove_content_type=True)
         self.refresh_session()
 
-    def relogin(self):
+    def relogin(self) -> None:
         """ Reconnect client cleaning headers and access_token. """
 
         # logout and login. Will fail if has no token
         logout_result = self.logout()
         login_result = self.login()
 
-    def get_settings(self):
+    def get_settings(self) -> dict:
         """ Get settings of netTime Server. """
 
         return self.get(path='/api/settings')
 
-    def get_user_rol(self):
+    def get_user_rol(self) -> str:
         """ Get user_rol of current session. """
 
         return self.settings.get('rol', None)
 
-    def get_days_offset(self, days: list):
+    def get_days_offset(self, days: list) -> list:
         """ 
         Convert a list ofdt.date or str format to int offset with 
         self.setting.firstDate.
@@ -374,7 +268,7 @@ class Client:
 
         return days_numbers
 
-    def get_days_from_offsets(self, offsets: list):
+    def get_days_from_offsets(self, offsets: list) -> list:
         """ 
         Convert a list of int offset todt.date or str format with \
         self.setting.firstDate.
@@ -499,7 +393,7 @@ class Client:
         return employees
 
     def container_action_exec(self, container: str, action: str, \
-            elements: list, _all: bool = False, dataObj: dict = None, \
+            elements: list, all_: bool = False, dataObj: dict = None, \
             *args, **kwargs):
         """ Execute an action for a container. """
 
@@ -507,7 +401,7 @@ class Client:
         json_data = {
             "container": container,
             "action": action,
-            "all": _all,
+            "all": all_,
             "elements": elements,
             "dataObj": dataObj
         }
@@ -517,7 +411,7 @@ class Client:
         return self.post(path='/api/container/action/exec', json=json_data)
 
     def save_element(self, container: str, dataObj: dict, \
-            elements: list = [], _all: bool = False):
+            elements: list = [], all_: bool = False):
         """ Update an element of a container with the received values. """
 
         # data prepare
@@ -525,7 +419,7 @@ class Client:
             "action": "Save",
             "container": container,
             "elements": elements,
-            "_all": _all,
+            "all_": all_,
             "dataObj": dataObj,
         }
 
@@ -533,7 +427,7 @@ class Client:
         return self.container_action_exec(**data)
 
     def delete_element(self, container: str, elements: list, \
-            _confirm: bool = True, _all: bool = False):
+            confirm_: bool = True, all_: bool = False):
         """ Delete an element of a container with the received values. """
 
         # data prepare
@@ -541,20 +435,20 @@ class Client:
             "action": "Delete",
             "container": container,
             "elements": elements,
-            "_all": _all,
+            "all_": all_,
         }
 
         # default auto confirm
-        if _confirm:
+        if confirm_:
             data["dataObj"] = {
-                "_confirm": _confirm,
+                "confirm_": confirm_,
             }
 
         # executing and processing
         return self.container_action_exec(**data)
 
     def get_for_duplicate(self, container: str, element: int, \
-            _all: bool = False):
+            all_: bool = False):
         """ Get form for a new element with data of recived element. """
 
         # data prepare
@@ -562,7 +456,7 @@ class Client:
             "action": "Copy",
             "container": container,
             "elements": [element],
-            "_all": _all,
+            "all_": all_,
         }
 
         # executing and processing
@@ -577,14 +471,14 @@ class Client:
 
 
     def get_element_def(self, container: str, elements: list, \
-            _all: bool = False, read_only: bool = False, *args, **kwargs):
+            all_: bool = False, read_only: bool = False, *args, **kwargs):
         """ Get all properties of an object/s. """
 
         # data prepare
         data = {
             "container": container,
             "elements": elements,
-            "_all": _all,
+            "all_": all_,
             "action": "editForm" if not read_only else "View",
         }
         data.update(kwargs)
@@ -616,7 +510,7 @@ class Client:
         return response[0]
 
     def get_day_info(self, employee: int, \
-            _from: str =dt.date.today().isoformat(), \
+            from_: str =dt.date.today().isoformat(), \
             to: str =dt.date.today().isoformat()):
         """ 
         Get info, days, shifts, and results for a employe in a specific period. 
@@ -625,7 +519,7 @@ class Client:
         # prepare task parameters
         params = {
             "idemp": employee,
-            "from": _from,
+            "from": from_,
             "to": to
         }
 
@@ -633,14 +527,14 @@ class Client:
         return self.get(path='/api/day/results', params=params)
 
     def get_access_clockings(self, employee: int, \
-            _from: str =dt.date.today().isoformat(), \
+            from_: str =dt.date.today().isoformat(), \
             to: str =dt.date.today().isoformat()):
         """ Get access clockings for a employe in a specific period. """
 
         # prepare task parameters
         params = {
             "idemp": employee,
-            "from": _from,
+            "from": from_,
             "to": to
         }
 
@@ -675,14 +569,14 @@ class Client:
         return self.get(path='/api/async/response', params=params)
 
     def get_results(self, employee: int, \
-            _from: str =dt.date.today().isoformat(), \
+            from_: str =dt.date.today().isoformat(), \
             to: str =dt.date.today().isoformat()):
         """ Get results of day for a employee. """
 
         # prepare task parameters
         params = {
             'idemp': employee,
-            'from': _from,
+            'from': from_,
             'to': to,
         }
 
@@ -847,13 +741,13 @@ class Client:
         }
         return self.save_element(**data)
 
-    def get_activity_monitor(self, employees: list, _from: str, to: str):
+    def get_activity_monitor(self, employees: list, from_: str, to: str):
         """ Return the activity monitor structure. """
         
         # prepare task parameters
         json_data = {
             'clockings': True,
-            'from': _from,
+            'from': from_,
             'to': to,
             'ids': employees
         }
