@@ -1,12 +1,20 @@
-from concurrent.futures import ThreadPoolExecutor, as_completed
-from urllib.parse import urlparse, urljoin
-from base64 import b64encode, b64decode
+from __future__ import annotations
+# from json.decoder import JSONDecodeError
+# from concurrent.futures import ThreadPoolExecutor, as_completed
+# from urllib.parse import urlparse, urljoin
+from pathlib import Path
+from typing import Any, Dict, List, Optional, Union
+from base64 import b64decode #, b64encode
 import requests
 import datetime
 import math
 
-
-# TODO: Change to use requests sessions
+try:
+    # from ._utils import Decorators #, create_random_suffix
+    from ._base import OAuthClient
+except ImportError:
+    # from _utils import Decorators #, create_random_suffix
+    from _base import OAuthClient
 
 
 class JsonObject:
@@ -16,7 +24,9 @@ class JsonObject:
         for k, v in json.items():
             setattr(self, k, v)
 
-class Client:
+class Client(OAuthClient):
+
+    __name__ = "Visma"
 
     class Account:
         def __init__(self, **kwargs):
@@ -25,15 +35,15 @@ class Client:
             self.roles = [JsonObject(roles) for roles in kwargs.get('roles')]
 
         @property
-        def ft_id(self):
+        def first_tenant_id(self):
             """ Return the id of the first available tenant. """
             return getattr(self.tenants[0], 'Id', None)
 
-        def get_tenant_id(self, _filters: dict):
+        def get_tenant_id(self, filters_: dict):
             """
             Get tenant ID from tenant name. Use to use specific tenant.
             
-            :param _filters: Dict with parameters to eval. Eg.
+            :param filters_: Dict with parameters to eval. Eg.
                 {"DBName": "Name_Of_Database", "TenantName": "Tenant_Test"}.
             :param tenant_name: (optional) Str with name of tenant.
             :param \*\*kwargs: Optional arguments to filter.
@@ -48,7 +58,7 @@ class Client:
                 _m = []
 
                 # eval all key: value parameters
-                for k, v in _filters.items():    
+                for k, v in filters_.items():    
                     _m.append(True if getattr(_t, k, None) == v else False)
 
                 # return tenant if all evals match
@@ -85,80 +95,88 @@ class Client:
         def is_expired(self):
             return not self.is_alive
 
-    def __init__(self, url: str, username: str, pwd: str, *args, **kwargs):
-        """ Create a conection with visma app using recived parameters. """
+    def __init__(
+        self,
+        *,
+        url: Union[str, Path],
+        username: str,
+        pwd: str,
+        session: Optional[requests.Session] = None,
+        tenant_filter: Optional[Dict[str, Any]] = None,
+    ) -> None:
+        """ Create a conection with visma app using recived parameters.
 
-        self.client_url = urlparse(url)
-        self.username = username
-        self.pwd = b64encode(pwd.encode('utf-8'))
+        Args:
+            url (Union[str, Path]): Visma URL.
+            username (str): Visma User
+            pwd (str): Visma Password
+            session (Optional[requests.Session], optional):
+                Optional session handler. Defaults to None.
+            tenant_filter (Optional[Dict[str, Any]], optional):
+                Optional filters to get tenant. Defaults to None.
+        """
 
-        self.authentication = None
-        self.account = None
-
-        # connect client and set authentication object automatically
-        self.connect()
-
-        # set account data automatically
-        self.account = self.Account(
-            tenants=self.get(path='/Admin/account/tenants'),
-            user_info=self.get(path='/Admin/account/user-info'),
-            roles=self.get(path='/Admin/account/roles')
+        super().__init__(
+            url=url,
+            username=username,
+            pwd=pwd,
+            session=session
         )
 
         # dict to filter tenant
-        self.tenant_filter = kwargs.get('tenant_filter', None)
-
-    def __str__(self):
-        return '{}{} en {}'.format(
-            f'{self.access_token} para ' if self.access_token else '',
-            self.username,
-            self.client_url.geturl()
-        )
-
-    def __repr__(self):
-        return "{}(url='{}', username='{}', pwd='{}')".format(
-            self.__class__.__name__,
-            self.client_url.geturl(),
-            self.username,
-            b64decode(self.pwd).decode('utf-8'),
-        )
-
-    def __enter__(self, *args, **kwargs):
-        return self
-
-    def __exit__(self, *args, **kwargs):
-        return self.disconnect()
-
-    @property
-    def headers(self):
-        """ Get headers of the client with current data """
-
-        # empty headers initial
-        data = {
+        self.tenant_filter = tenant_filter
+        
+        # base headers
+        self.headers = {
             "Accept": "application/json",
             "Accept-Encoding": "gzip,deflate",
         }
 
-        # logged user
+        self.authentication = None
+        self.account = None
+        self.tenant = None
+
+    def refresh_headers(
+        self,
+        auth_data: Optional[Dict[str, Any]] = None,
+        tenant: Optional[Union[int, str]] = None,
+        remove_auth: Optional[bool] = False,
+        remove_content_type: Optional[bool] = False
+    ) -> None:
+        if remove_auth:
+            self.authentication = None
+            self.session.headers.pop("Authorization", None)
+
+        if remove_content_type:
+            self.headers.pop("Content-Type", None)
+            self.session.headers.pop("Content-Type", None)
+        
+        if auth_data:
+            self.authentication = self.Authentication(**auth_data)
+
+        if tenant:
+            self.headers.update({'X-RAET-Tenant-Id': tenant})
+            self.session.headers.update({'X-RAET-Tenant-Id': tenant})
+
         if self.authentication:
-            data["Authorization"] = str(self.authentication)
+            self.headers.update({
+                "Authorization": str(self.authentication)
+            })
+            self.session.headers.update({
+                "Authorization": str(self.authentication)
+            })
 
-        # tenant obtained
-        if self.account:
-            if self.tenant_filter:
-                # get id with filter dict
-                data["X-RAET-Tenant-Id"] = self.account.get_tenant_id(
-                    _filters=self.tenant_filter
-                )
-            else:
-                # get first available tenant
-                data["X-RAET-Tenant-Id"] = getattr(
-                    self.account,
-                    'ft_id',
-                    None
-                )
+        if "Content-Type" not in self.headers and not remove_content_type:
+            self.headers.update({
+                "Content-Type": "application/json;charset=UTF-8"
+            })
+            self.session.headers.update({
+                "Content-Type": "application/json;charset=UTF-8"
+            })
 
-        return data
+    @property
+    def access_token(self) -> Union[str, None]:
+        return getattr(self.authentication, 'access_token', None)
     
     @property
     def is_connected(self):
@@ -174,192 +192,78 @@ class Client:
 
         return getattr(self.authentication, 'is_expired', None)
 
-    def get(self, path: str, params: dict = None, **kwargs):
-        """
-        Sends a GET request to visma url.
+    def set_account(self) -> None:
+        self.account = self.Account(
+            tenants=self.get(path='/Admin/account/tenants'),
+            user_info=self.get(path='/Admin/account/user-info'),
+            roles=self.get(path='/Admin/account/roles')
+        )
 
-        :param path: path to add to URL for the new :class:`Request` object.
-        :param params: (optional) Dictionary, list of tuples or bytes to send
-            in the query string for the :class:`Request`.
-        :param \*\*kwargs: Optional arguments that ``request`` takes.
-        :return: :class:`dict` object
-        :rtype: dict
-        """
-
-        # check if session has expired
-        if self.session_expired:
-            self.reconnect()
-
-        # safety only
-        if not self.is_connected and not kwargs.get('force', None):
-            raise ConnectionError("Cliente desconectado. Utilice connect().")
-
-        # query prepare
-        query = {
-            "url": urljoin(self.client_url.geturl(), path),
-            "params": params,
-            "headers": self.headers,
-            "timeout": kwargs.get("timeout", 60),
-            "stream": kwargs.get("stream", False)
-        }
-
-        # consulting nettime
-        response = requests.get(**query)
-
-        # raise if was an error
-        if response.status_code not in range(200, 300):
-            raise ConnectionError(response.text)
-
-        # to json -> json
-        try:
-            # if request is stream type, return all response
-            if kwargs.get("stream"):
-                return response
-
-            # return json response
-            return response.json()
-        except:
-            return {}
-
-    def bulk_get(self, path: str, params: dict = {}, \
-            together: tuple = tuple(), max_workers: int = 50, \
-            response_to_dict: bool = False, **kwargs):
-        """
-        Get from client object simultaneously.
+    def login(self) -> None:
         
-        :param path: path to add to URL for the new :class:`Request` object.
-        :param params: (optional) Dictionary, list of tuples or bytes to send
-            in the query string for the :class:`Request`.
-        :param together: Iterator of dicts to get together. E.g. 
-            [{
-                'page': 1
-            },{
-                'page': 2
-            }]
-        :max_workers: int to specify max workers number to run simultaneously.
-        :response_to_dict: bool to group responses if there are dict objects.
-        :param \*\*kwargs: Optional arguments that ``request`` takes.
-        :return: :class:`dict` object
-        :rtype: dict
-        """
-
-        # structures prepare
-        responses = {} if response_to_dict else []
-        processes = []
-        params_ = params
-
-        # create executor context
-        with ThreadPoolExecutor(max_workers=max_workers) as executor:
-            # processes prepare
-            for e_ in together:
-                params_.update(e_)
-                processes.append(
-                    executor.submit(
-                        self.get,
-                        path=path,
-                        params=params_,
-                        **kwargs
-                    )
-                )
-
-            # execute and put result in out structure
-            if response_to_dict:
-                [responses.update(r.result()) for r in as_completed(processes)]
-            else:
-                [responses.append(r.result()) for r in as_completed(processes)]
-
-            # close executor
-            executor.shutdown(wait=True)
-
-        # return responses
-        # list of pages result
-        return responses
-
-    def post(self, path, data=None, json=None, **kwargs):
-        """
-        Sends a POST request to visma url.
-
-        :param url: URL for the new :class:`Request` object.
-        :param data: (optional) Dictionary, list of tuples, bytes, or file-like
-            object to send in the body of the :class:`Request`.
-        :param json: (optional) json data to send in the body of the 
-            :class:`Request`.
-        :param \*\*kwargs: Optional arguments that ``request`` takes.
-        :return: :class:`dict` object
-        :rtype: dict
-        """
-
-        # check if session has expired
-        if self.session_expired:
-            self.reconnect()
-
-        # wait active conection
-        if not self.is_connected and not kwargs.get('force', None):
-            raise ConnectionError("Cliente desconectado. Utilice connect().")
-
-        # query prepare
-        query = {
-            "url": urljoin(self.client_url.geturl(), path),
-            "data": data,
-            "json": json,
-            "headers": self.headers,
-            "timeout": kwargs.get("timeout", 20),
-        }
-
-        # consulting nettime
-        response = requests.post(**query)
-
-        # raise if was an error
-        if response.status_code not in range(200, 300):
-            raise ConnectionError(response.text)
-
-        # to json -> json
-        try:
-            return response.json()
-        except:
-            return {}
-
-    def connect(self):
-        """ Connect the client to get access_token and headers values. """
-
-        # None or not is_alive
         if self.is_connected:
             return
 
-        # url and data prepare
+        # remove content type from headers. Must be not json
+        self.refresh_headers(remove_auth=True, remove_content_type=True)
+
+        # data prepare
         data = {
             "username": self.username,
             "password": b64decode(self.pwd).decode('utf-8'),
             "grant_type": "password"
         }
-        response = self.post(
-            path='/Admin/authentication/login',
-            data=data,
-            force=True
-        )
 
-        # if everything ok
-        self.authentication = self.Authentication(**response)
+        # consulting nettime
+        json_data = self.post(path='/Admin/authentication/login', data=data)
 
-    def disconnect(self):
-        """ Disconnect the client if is connected. """
+        # self.authentication = self.Authentication(**json_data)
+        self.refresh_headers(auth_data=json_data)
+        self.refresh_session()
+
+        # update tenant
+        self.set_account()
+        if self.tenant_filter:
+            self.tenant = self.account.get_tenant_id(
+                filters_=self.tenant_filter
+            )
+        else:
+            self.tenant = getattr(
+                self.account,
+                'first_tenant_id',
+                None
+            )
+
+        self.refresh_headers(tenant=self.tenant)
+        self.refresh_session()
+
+    def logout(self) -> None:
+        """ Disconnect a client to clean the access_token. """
 
-        # None or not is_alive
         if not self.is_connected:
             return
 
+        # disconnect ...
         response = self.post(path='/Admin/authentication/logout')
-        self.authentication = None
 
-    def reconnect(self):
+        self.refresh_headers(remove_auth=True, remove_content_type=True)
+        self.refresh_session()
+
+    def relogin(self):
         """ Reconnect client cleaning headers and access_token. """
 
-        # clean token for safety
-        self.authentication = None
-        self.connect()
+        # logout and login. Will fail if has no token
+        logout_result = self.logout()
+        login_result = self.login()
 
-    def get_employees(self, employee: str = None, extension: str = None, \
-            all_pages: bool = False, bulk_get: bool = False, **kwargs):
+    def get_employees(
+        self,
+        employee: Optional[Union[str, int]] = None,
+        extension: Optional[str] = None,
+        all_pages: bool = False,
+        bulk_get: bool = False,
+        **kwargs
+    ):
         """
         Use the endpoint to obtain the employees with the received data.
         
@@ -548,7 +452,7 @@ class Client:
 
         # path prepare
         path = '/WebApi/leaves{}'.format(
-            f'/{extension}' if journal and extension else '',
+            f'/{extension}' if extension else '',
         )
 
         # getting default date
@@ -678,13 +582,8 @@ class Client:
         # path prepare
         path = '/WebApi/pay-elements/individual'
 
-        # json prepare
-        _json = {
-            "values": values
-        }
-
         # request.get -> json
-        return self.post(path=path, json=_json, **kwargs)
+        return self.post(path=path, json={"values": values}, **kwargs)
 
     def get_payments(self, extension: str, **kwargs):
 
